#ifndef nsDocumentCharsetInfoCID_h__
#define nsDocumentCharsetInfoCID_h__

#include "nscore.h"

#define NS_DOCUMENTCHARSETINFO_CONTRACTID "@mozilla.org/document-charset-info;1"

// {D25E0511-2BAE-11d4-9D10-0050040007B2}
#define NS_DOCUMENTCHARSETINFO_CID \
  {0xd25e0511, 0x2bae, 0x11d4, {0x9d, 0x10, 0x0, 0x50, 0x4, 0x0, 0x7, 0xb2}}

#endif // nsDocumentCharsetInfoCID_h__

